unit Main;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  GL,
  GLU,
  GLUT,
  GLClass,
  GLTexture,
  GLRenderToVideo,
  Dialogs,
  ExtCtrls,
  ComCtrls, AppEvnts;

type
  TMainForm = class(TForm)
    Timer: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
  public
    { Public declarations }
    //Declare an instance so we can use it
    GL: TOpenGL;
    //we need to set up a projection matrix, right?
    ViewVolume: TViewVolume;

    //The texture
    tex: TTexture;

    //Variables to store time
    DemoStart : cardinal;
    ElapsedTime : cardinal;

    //Capture has started?
    Capturing : boolean;

    // The capture class
    vid: TGLRenderToVideo;

    temp: TBitmap;

    procedure RenderCube;
    procedure GetScreenBitmap(var aBitmap:TBitmap);
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

procedure TMainForm.RenderCube;
begin
  glBegin(GL_QUADS);
    // Front Face
    glNormal3f(0.0, 0.0, 0.5);
    glTexCoord2f(0.0, 0.0);
    glVertex3f(-1.0, -1.0, 1.0);
    glTexCoord2f(1.0, 0.0);
    glVertex3f(1.0, -1.0, 1.0);
    glTexCoord2f(1.0, 1.0);
    glVertex3f(1.0, 1.0, 1.0);
    glTexCoord2f(0.0, 1.0);
    glVertex3f(-1.0, 1.0, 1.0);

    // Back Face
    glNormal3f(0.0, 0.0, -0.5);
    glTexCoord2f(1.0, 0.0);
    glVertex3f(-1.0, -1.0, -1.0);
    glTexCoord2f(1.0, 1.0);
    glVertex3f(-1.0, 1.0, -1.0);
    glTexCoord2f(0.0, 1.0);
    glVertex3f(1.0, 1.0, -1.0);
    glTexCoord2f(0.0, 0.0);
    glVertex3f(1.0, -1.0, -1.0);

    // Top Face
    glNormal3f(0.0, 0.5, 0.0);
    glTexCoord2f(0.0, 1.0);
    glVertex3f(-1.0, 1.0, -1.0);
    glTexCoord2f(0.0, 0.0);
    glVertex3f(-1.0, 1.0, 1.0);
    glTexCoord2f(1.0, 0.0);
    glVertex3f(1.0, 1.0, 1.0);
    glTexCoord2f(1.0, 1.0);
    glVertex3f(1.0, 1.0, -1.0);

    // Bottom Face
    glNormal3f(0.0, -0.5, 0.0);
    glTexCoord2f(1.0, 1.0);
    glVertex3f(-1.0, -1.0, -1.0);
    glTexCoord2f(0.0, 1.0);
    glVertex3f(1.0, -1.0, -1.0);
    glTexCoord2f(0.0, 0.0);
    glVertex3f(1.0, -1.0, 1.0);
    glTexCoord2f(1.0, 0.0);
    glVertex3f(-1.0, -1.0, 1.0);

    // Right face
    glNormal3f(0.5, 0.0, 0.0);
    glTexCoord2f(1.0, 0.0);
    glVertex3f(1.0, -1.0, -1.0);
    glTexCoord2f(1.0, 1.0);
    glVertex3f(1.0, 1.0, -1.0);
    glTexCoord2f(0.0, 1.0);
    glVertex3f(1.0, 1.0, 1.0);
    glTexCoord2f(0.0, 0.0);
    glVertex3f(1.0, -1.0, 1.0);

    // Left Face
    glNormal3f(-0.5, 0.0, 0.0);
    glTexCoord2f(0.0, 0.0);
    glVertex3f(-1.0, -1.0, -1.0);
    glTexCoord2f(1.0, 0.0);
    glVertex3f(-1.0, -1.0, 1.0);
    glTexCoord2f(1.0, 1.0);
    glVertex3f(-1.0, 1.0, 1.0);
    glTexCoord2f(0.0, 1.0);
    glVertex3f(-1.0, 1.0, -1.0);
  glEnd();
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  //Create it. We are going to render to the form.
  GL := TOpenGL.Create(Self);
  GL.FullScreen := false;
  GL.XRes := 320;
  GL.YRes := 240;

  ClientWidth := GL.XRes;
  ClientHeight := GL.YRes;

  GL.InitGL;

  //That's it. Cool, aint it?
  //Any changes you want to make will never muck up this code.
  GL.InitGL;

  //Create it and set up the width and height of the window
  ViewVolume := TViewVolume.Create;
  //we could also do this if we wanted to resize the window
  ViewVolume.Width := GL.XRes;
  ViewVolume.Height := GL.YRes;

  glEnable(GL_DEPTH_TEST);
  glEnable(GL_TEXTURE_2D);
  glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
  glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

  //create the texture
  tex := TTexture.Create;
  //Load the TGA
  tex.LoadTexture(ExtractFilePath(ParamStr(0))+'fan3.jpg');
  //Set up the blend function
  tex.Opaque;

  DemoStart := GetTickCount;

  Capturing := false;
  vid := TGLRenderToVideo.Create;
  vid.Filename := ExtractFilePath(ParamStr(0))+'test.avi';
  vid.Rate := 24;
  temp := TBitmap.Create;
  temp.Width := ClientWidth;
  temp.Height := ClientHeight;

  //Let the flipping begin
  Timer.Enabled := true;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  //Stop the rendering. We dont want to try and render
  //a destroyed object, right?!!
  Timer.Enabled := false;

  //Stop capturing
  if Capturing then
    vid.EndCapture;
  // Free the capture class 
  vid.Free;
  // Free the bitmap here
  temp.Free;

  if assigned(GL) then
    GL.Destroy;
end;

procedure TMainForm.TimerTimer(Sender: TObject);
begin
  //Clear contents of previous render
  GL.ClearAll;

  //Set the projection matrix
  ViewVolume.SetMatrix;

  //Calculate how many ticks snce the prog started
  ElapsedTime := GetTickCount - DemoStart;

  //We will simply translate ourselves and watch a little cube
  glTranslatef(0, 0, -10);
  glRotatef(ElapsedTime / 50, 1, 0, 0);
  glRotatef(ElapsedTime / 60, 0, 1, 0);

  tex.Opaque;
  tex.Enable;
  RenderCube;

  //Flip the screen
  GL.Flip;

  // Are we capturing?
  if Capturing then
  begin
    // If so, stop the timer, and add the current frame to the AVI
    Timer.Enabled := false;
    GetScreenBitmap(temp);
    vid.AddFrame(temp);
    Timer.Enabled := true;
  end;
end;

procedure TMainForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  case Key of
    //Now quit, we should...like any good app would :)
    VK_ESCAPE: Close;
    VK_F1:begin
      Capturing := not Capturing;
      if not Capturing then
      begin
        // Toggle the status
        Caption := 'Not capturing....';
        // Stop capturing
        vid.EndCapture;
      end;
      if Capturing then
      begin
        Caption := 'Capture in progress...';
        // Get the first frame
        GetScreenBitmap(temp);

        // Start the capture
        vid.BeginCapture(temp);
      end;
    end;
  end;
end;

procedure TMainForm.GetScreenBitmap(var aBitmap: TBitmap);
begin
  // Of course, you could use glReadPixels here.....
  aBitmap.Canvas.CopyRect(aBitmap.Canvas.ClipRect, Self.Canvas, Bounds(0, 0, ClientWidth, ClientHeight));
end;

end.

